function [A, B] = equator2horizon(RA, DEC, Lambda, Phi, GAST)

%EQUATOR2HORIZON convert equatorial to horizontal coordinates 
%
%SYNOPSIS
%   [A, B] = equator2horizon(RA, DEC, LAMBDA, PHI, GAST)
%
%   The equatorial coordinates of a star are given by its right 
%   ascention (RA) and declination (DEC). The horizontal coordinates 
%   are altitude and azimuth. Azimuth is the angle measured clockwise 
%   along the horizon from the North to the intersection with a great 
%   circle drawn from the zenith through the star. Altitude is the 
%   angle of a star above the horizon, measured along a great circle 
%   passing through the star and the zenith. For details see 
%   http://aa.usno.navy.mil/faq/docs/Alt_Az.html
% 
%INPUTS
%    RA     - Right ascention of the star [hours]
%    DEC    - Declination of the star [degrees]
%    Lambda - Longitude of the observer [degrees]
%             West of Greenwich , Lambda < 0
%             East of Greenwich, Lambda > 0
%    Phi    - Latitude of the observer [degrees]
%    GAST   - Greenwich apparent sidereal time [hours]
%
%OUTPUTS
%   A - Altitude of the star [degrees]
%   B - Azimuth of the star [degress]
%
%AUTHOR
%   Shantanu Desai (No longer part of the LV Collaboration)

% $Id: equator2horizon.m 6645 2010-12-09 16:09:14Z diego.menendez@LIGO.ORG $

% calculate local hour angle (LHA)

LHA = (GAST - RA) * 15 + Lambda;    % LHA in degrees



% calculate A1, A2, A3

A1 =  sind(Phi) * sind(DEC) + cosd(Phi) * cosd(DEC) * cosd(LHA);
A2 =  cosd(Phi) * sind(DEC) - sind(Phi) * cosd(DEC) * cosd(LHA);
A3 = -cosd(DEC) * sind(LHA);


% calculate altitude (A) and azimuth (B)

A = asind(A1);
B = atan2(A3, A2);

B = B * 180/pi;
B = mod(B, 360);

return
