function list = loadsegments(file);

%LOADSEGMENTS Read list of segments from a file
%
% LOADSEGMENTS reads a segment list from the specified file, which
% should contain at least two columns of whitespace delimited ASCII
% text with either of the following formats.
%
%   start1 stop1        start1 length1
%   start2 stop2        start2 length2
%   start3 stop3   or   start3 length3
%    ...    ...          ...    ... 
%   startN stopN        startN lengthN
%
% The format is determined by the contents of the second column.  If all
% values in the second column are less than or equal to the corresponding
% start time, then all segments are assumed to be specified in terms of
% their start time and duration.
%
%SYNOPSIS
%   LIST = loadsegments(FILE);
%
%INPUT
%   FILE      filename to read
%
%OUTPUT
%   LIST      resulting segment list
%
% The resulting segments list will be in the form of an N-by-2 matrix,
% formed from adjacent column-wise vectors of segment start and stop
% times.
%
%   start1 stop1
%   start2 stop2
%   start3 stop3
%    ...    ...
%   startN stopN
%
% By definition, a segment is the half closed interval [start,stop).
% A segment is inclusive of the specified start time, but exclusive of
% the specified stop time.  The duration of a segment is defined as the
% difference between stop and start times.
%
% Lines starting with '#' are considered comment lines and are
% ignored.  Only the first two columns in a file are read, additional
% columns are ignored.
%
% See also LOADCONLOG and WRITESEGMENTS.
%
%AUTHOR
%   Shourov K. Chatterji (No longer part of the LV Collaboration)

% $Id: loadsegments.m 6645 2010-12-09 16:09:14Z diego.menendez@LIGO.ORG $

% parse command line arguments
error(nargchk(1, 1, nargin));

% read file
[start, stop] = textread(file, '%f%f%*[^\n]', 'commentstyle', 'shell');

% create output segment list
list = [start stop];

% standardize segment list
list = stdsegments(list);

% remove zero length segments
list = cullsegments(list);

% merge overlapping or adjacent segments
list = mergesegments(list);

% return to calling function
return;
