function [list, indices] = sortsegments(list, criteria);

%SORTSEGMENTS Sort segment lists by start, stop, or duration
%
% SORTSEGMENTS reads the requested segment list and sorts segments
% in increasing or decreasing order according to their start time,
% stop time, and or duration.  The resulting sorted segment list is
% returned as well as an index vector which maps the input segments
% list to the sorted output list.
%
% SORTSEGMENTS does not remove redundant segments.  This can be
% accomplished using UNIQUESEGMENTS or MERGESEGMENTS.  The order of
% redundant segments is preserved.
%
% The sort criteria are specified as a one or two element vector, and
% may contain the following values
%
%   1    start time
%   2    stop time
%   3    duration
%
% For example, the criteria [1 2] sorts in increasing order based
% first on start time and secondly on stop time.  This is also the
% default criteria, if none is specified.
%
% The sort order may be reversed for a particular criteria, by making
% that element of the criteria vector negative.
%
%SYNOPSIS
%   [OUT, INDEX] = sortsegments(IN, CRITERIA);
%
%INPUTS
%   IN        input segment list
%   CRITERIA  sort criteria
%
%OUTPUTS
%   OUT       output segment list
%   INDEX     sorted index into input list
%
% The index vector maps the input segment list to the sorted output
% segments list in the following way.
%
% out = in(index, :);
%
% The segments list should be provided in the form of an N-by-2 matrix,
% formed from adjacent column-wise vectors of segment start and stop
% times.
%
%   start1 stop1
%   start2 stop2
%   start3 stop3
%    ...    ...
%   startN stopN
%
% By definition, a segment is the half closed interval [start,stop).
% A segment is inclusive of the specified start time, but exclusive of
% the specified stop time.  The duration of a segment is defined as the
% difference between stop and start times.
%
% As an alternative, each segment may also be specified by its start time
% and duration.  However, the resulting segment list will be returned in
% terms of start and stop times.
%
% Due to the algorithm used, the efficiency of SORTSEGMENTS may
% decrease if a secondary criteria is requested and a high level of
% redundancy is encountered the first sort criteria.
%
% See also NUMSEGMENTS, MERGESEGMENTS, and UNIQUESEGMENTS.
%
%AUTHOR
%   Shourov K. Chatterji (No longer part of the LV Collaboration)

% $Id: sortsegments.m 6645 2010-12-09 16:09:14Z diego.menendez@LIGO.ORG $

% parse command line arguments
error(nargchk(1, 2, nargin));

% default criteria
if nargin == 1,
  criteria = [1 2];
end

% standardize segment list
list = stdsegments(list);

% handle empty lists
if isempty(list),
  indices = [];
  return;
end

% force row vector criteria
criteria = criteria(:)';

% verify valid criteria
if length(criteria) < 1,
  error('infuccient criteria');
elseif length(criteria) > 2,
  error('more than two criteria specified');
end

% extract start and stop vectors
start = list(:, 1);
stop = list(:, 2);

% get segment durations
duration = stop - start;

% master sort list
list = [start stop duration];

% organize sort list by criteria
list = list(:, abs(criteria)) .* (ones(length(start), 1) * sign(criteria));

% test for prior sorting
if length(criteria) == 2,
  issorted1 = all(diff(list(:, 1)) >= 0);
  issorted2 = all((diff(list(:, 1)) > 0) | ...
                  ((diff(list(:, 1)) == 0) & ...
                   (diff(list(:, 2)) >= 0)));
  issorted = issorted2;
else
  issorted1 = all(diff(list(:, 1)) >= 0);
  issorted =  issorted1;
end

% if already sorted,
if issorted,
  
  % generate result list
  list = [start stop];
  
  % generate trivial result indices
  indices = (1 : length(start))';

  % return
  return;

% end if already sorted
end

% if already sorted by first crieta
if issorted1,
  
  % create trivial index vector
  indices = (1 : length(start))';

% else, if not already sorted by first criteria
else

  % sort by first criteria
  [ignore, indices] = sort(list(:, 1));

% end if already sorted by first criteria
end

% if secondary sort criteria,
if length(criteria) == 2,

  % reorder sort list based on primary sort
  list = list(indices, :);

  % find non-unique subgroups
  sub_edges = diff([0; (diff(list(:, 1)) == 0); 0]);
  sub_start = find(sub_edges == 1);
  sub_stop = find(sub_edges == -1);

  % loop over subgroups
  for subgroup = 1 : length(sub_start),

    % index into subgroup
    sub_indices = sub_start(subgroup) : sub_stop(subgroup);
    
    % sort subgroup by second criteria
    [ignore, reorder] = sort(list(sub_indices, 2));
    
    % reorder indices based on sort
    indices(sub_indices) = indices(sub_indices(reorder));

  % end loop over subgroups
  end

% end sort on secondary criteria
end
  
% generate result list
list = [start stop];

% reorder result list based on sort
list = list(indices, :);

% standardize segment list
list = stdsegments(list);

% return to calling function
return;
