function out = splitsegments(in, maximum, overlap, minimum);

%SPLITSEGMENTS split up long segments into shorter overlapping segments
%
% SPLITSEGMENTS splits up segments from the given segment list which are
% longer than the specified maximum duration in seconds.  Such segments are
% made smaller by succesively selecting maximum duration subsegments until
% the final remain subsegment is also shorter than the specified maximum
% duration.  To compensate for filter transients, the split segments are
% forced to overlap by the specified overlap amount in seconds.  By
% default, the required overlap is zero.
%
% SPLITSEGMENTS also ensures that the subsegments are all longer than the
% specified minimum segment length.  This is accomplished by increasing the
% overlap in the last subsegment such that the minimum segment length is
% met.  However, SPLITSEGMENTS does not remove segments from the original
% list that are shorter than the minimum specified length.  This should be
% done using CULLSEGMENTS.  By default, the minimum subsegment length is
% zero.
% 
% Note that currently some segment processing functions automatically merge
% overlapping or contiguous segments.  As a result, SPLITSEGMENTS should be
% the last function that is applied.
%
% SPLITSEGMENTS returns the resulting segment list.
%
%SYNOPSIS
%   OUT = splitsegments(IN, MAXIMUM, OVERLAP, MINIMUM);
%
%INPUTS
%   IN         input segment list
%   MAXIMUM    maximum segment length in seconds
%   OVERLAP    required segment overlap in seconds
%   MINIMUM    minimum segment length in seconds
%
%OUTPUT
%   OUT        output segment list
%
% The segments list should be provided in the form of an N-by-2 matrix,
% formed from adjacent column-wise vectors of segment start and stop
% times.
%
%   start1 stop1
%   start2 stop2
%   start3 stop3
%    ...    ...
%   startN stopN
%
% By definition, a segment is the half closed interval [start,stop).
% A segment is inclusive of the specified start time, but exclusive of
% the specified stop time.  The duration of a segment is defined as the
% difference between stop and start times.
%
% As an alternative, each segment may also be specified by its start time
% and duration.  However, the resulting segment list will be returned in
% terms of start and stop times.
%
% See also MERGESEGMENTS, LENSEGMENTS, and CULLSEGMENTS.
%
%AUTHOR
%   Shourov K. Chatterji (No longer part of the LV Collaboration)

% $Id: splitsegments.m 6645 2010-12-09 16:09:14Z diego.menendez@LIGO.ORG $

% parse command line arguments
error(nargchk(2, 4, nargin));

% default segment overlap
if nargin == 2,
  overlap = 0;
  minimum = 0;
elseif nargin == 3,
  minimum = 0;
end

% validate parameters
if overlap >= maximum,
  error('requested overlap >= requested maximum');
end
if minimum >= maximum,
  error('requested minimum >= requested maximum');
end

% standardize segment list
in = stdsegments(in);

% handle empty lists
if isempty(in),
  out = in;
  return;
end

% initialized output segment list
out = zeros(0, 2);

% begin loop over input segment list
for index = 1 : size(in, 1),

  % extract input segment start time
  start = in(index, 1);

  % extract input segment stop time
  stop = in(index, 2);

  % determine input segment duration
  duration = stop - start;

  % if segment duration does not exceed limit
  if duration <= maximum,

    % append segment to output
    out = [out; start stop;];

    % continue to next segment
    continue;

  % otherwise, split the segment
  end

  % vector of start times for subsegments
  substarts = start : maximum - overlap : stop - maximum;

  % vector of stop times for subsegments
  substops = substarts + maximum;

  % fix start time of last subsegment
  if substops(end) < stop,
    substarts = [substarts ...
                 min(stop - minimum, substops(end) - overlap)];
    substops = [substops stop];
  end

  % append new subsegments to output
  out = [out; substarts' substops';];

% end loop over input segment list
end

% return to calling function
return;
