function [h,s,p,cp,ulrate,pulrate] = bayesRate(b,N,T,fig,ulim,ulerr)

%BAYESRATE posterior prob. of signal rate of  Poisson proc. in a Poisson bkg
%
%SYNOPSIS
%   [H,S,P,CP] = bayesRate(B,N,T,FIG)
%
%INPUTS
%   b     - background rate
%   N     - number events observed
%   T     - observation duration (seconds)
%   fig   - true: displayed figure of posterior probability density,
%           cumulative probability
%   ulim  - upper limit based on cp< ulim (default 0.95)
%   ulerr - accuracy at which point to stop search for upperlimit (0.001)
%
%OUTPUTS
%   h     - figure handle 
%   s     - signal rate
%   p     - posterior probability density P(s)
%   cp    - posterior cumulative probability P(<s)
%   ulrate- upper-limit rate pair (low,high)
%   pulrate- posterior cumulative probability at ulrate
%
%AUTHOR
%   Keith Thorne <keith.thorne@ligo.org>

% $Id: bayesRate.m 6645 2010-12-09 16:09:14Z diego.menendez@LIGO.ORG $

% See T050022
% Note that Matlab has different conventions for the incomplete Gamma
% function then you will find most other places. In particular, the most
% common notation is 
%
% \Gamma(a,z) = \int_z^\infty x^(a-1) e^{-x} dx
%
% In matlab,
%
% gammainc(z,a,'upper') = \Gamma(a,z)/\Gamma(a,0)
%
% This leads to a problem when a=0. Fortunately, when a=0
%
% \Gamma(0,z) = \int_z^\infty dx \exp(-x)/x
%
% which is the exponential integral,
%
% expint(z) = \Gamma(0,z)
%
% We use the exponential integral function in the special case N = 0. 

error(nargchk(4,6,nargin),'struct');
if nargin<5
  ulim=0.95;
end
if nargin<6
  ulerr=0.001;
end
h = [];
s = [];
p = [];
cp = [];
ulrate = [];
pulrate = [];

% Add test that time period is non-zero
if(T <= 0)
    h = [];
    s = [];
    p = [];
    cp = [];
    return
end
Nmax = N + sqrt(max([N,16]));
s = linspace(0,Nmax,200)/T;
[p,cp]=calcBayesRate(b,N,T,s);
% Now form cumulative probability
if ~fig
  figure('visible','off');
else
  figure('visible','on');
end
[ax,h1,h2] = plotyy(s,p,s,cp);

set(get(ax(1),'Ylabel'),'String','P(s|N,b,T)');
set(get(ax(2),'Ylabel'),'String','P(<s|N,b,T)');
xlabel('Signal rate [Hz]');

axes(ax(2));
hold on;
xmax = max(s);
xmin = min(s);
plot([xmin xmax],[ulim ulim],'r-.','LineWidth',3);
hold off;
h = gcf;

cpt=cp; %temp
st=s;

%find upperlimit
if ulim>0
  ulow=max(find(cpt<ulim));
  uhigh=min(find(cpt>=ulim));
  while cpt(uhigh)<ulim
    st = [s(uhigh),2*s(uhigh)];
    [pt,cpt]=calcBayesRate(b,N,T,st);
    ulow=max(find(cpt<ulim));
    uhigh=min(find(cpt>=ulim));
  end
  while(cpt(uhigh)-cpt(ulow)>ulerr)
    st = [st(ulow),(st(ulow)+st(uhigh))/2,st(uhigh)];
    [pt,cpt]=calcBayesRate(b,N,T,st);
    ulow=max(find(cpt<ulim));
    uhigh=min(find(cpt>=ulim));
  end
  ulrate=[st(ulow),st(uhigh)];
  pulrate=[cpt(ulow),cpt(uhigh)];
end

return

function [p,cp]=calcBayesRate(b,N,T,s)
%still have problem if for very small pnorm where roundoff -> pnorm=0
n = (s+b)*T;
if N > 0
  pnorm = gammainc(b*T,N,'upper');
  pnorm = max(pnorm,eps(pnorm));
  p = T * exp(-n) .* (n .^ (N-1))/(pnorm*gamma(N));
  cp = pnorm-gammainc(n,N,'upper');
  cp = cp/pnorm;
else
%  Limit the background rate to at least 0.001 over the interval
%  for calculating probabilities
  bkgdLim = 0.001 / T;
  b = max(b,bkgdLim);
%  add limit to ensure pnorm, n stay above 0
%    due to division
  n = max(n,eps(n(1)));
%  
  pnorm = expint(b*T);
  pnorm = max(pnorm,eps(pnorm));
  p = T * exp(-n) ./ (n .* pnorm);
  cp = (expint(b*T)-expint(n))/pnorm;
end
return
